import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_form_appbar.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_test_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:cramr_ui_lib/widgets/conjugation_meta_data_card.dart';
import 'package:cramr_ui_lib/widgets/coverable_textfield.dart';
import 'package:cramr_ui_lib/widgets/pronoun_field.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// The Route that allows you to study the indicative of the verb with the cover
/// mode.
class VerbIndicativeStudyCoverModeRoute extends StatelessWidget {
  static const routeName = '/verbIndicativeCoverModeStudyRoute';

  final SettingsController _settingsController;

  const VerbIndicativeStudyCoverModeRoute(this._settingsController,
      {required Key key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: _settingsController,
        builder: (BuildContext context, Widget? child) {
          return MaterialApp(
            title: "Verbomatique",
            themeMode: _settingsController.themeMode,
            theme: CustomTheme.lightTheme,
            darkTheme: ThemeData.dark(),
            debugShowCheckedModeBanner: false,
            home: SafeArea(
              key: UniqueKey(),
              child: Scaffold(
                appBar: VerbIndicativeStudyFormAppbar().asWidget(context),
                body:
                    Container(key: UniqueKey(), child: _centerWidget(context)),
                floatingActionButton: _toTestRouteButton(context),
              ),
            ),
          );
        });
  }

  Widget _centerWidget(BuildContext context) {
    return Column(
      children: [
        _conjugationMetaDataCard(context),
        const Divider(),
        Expanded(child: _listView(context)),
      ],
    );
  }

  Widget _conjugationMetaDataCard(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return ConjugationMetaDataCard(
          infinitive: testModel.infinitive,
          translation: testModel.translation,
          mode: testModel.mode,
          tense: testModel.tense);
    });
  }

  Widget _toTestRouteButton(BuildContext context) {
    return FloatingActionButton(
      key: toTestButtonKey(),
      child: const Icon(Icons.arrow_forward),
      onPressed: () {
        final StudyModeState studyModeState =
            Provider.of<StudyModeState>(context, listen: false);
        studyModeState.updateState(StudyMode.coverMode);
        Navigator.pushNamed(context, VerbIndicativeTestRoute.routeName);
      },
    );
  }

  Widget _listView(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return ListView.builder(
          key: Key('study_scroll_${testModel.infinitive}'),
          itemCount: testModel.pronouns.length,
          itemBuilder: (BuildContext context, int index) {
            return Padding(
              key: UniqueKey(),
              padding: const EdgeInsets.fromLTRB(20, 0, 5, 0),
              child: Card(
                key: UniqueKey(),
                clipBehavior: Clip.antiAlias,
                child: ListTile(
                  key: UniqueKey(),
                  visualDensity: VisualDensity.comfortable,
                  contentPadding: const EdgeInsets.fromLTRB(5, 2, 5, 2),
                  dense: true,
                  leading: _pronounCell(context, index),
                  title: CoverableTextField(person: index),
                ),
                elevation: 5.0,
              ),
            );
          });
    });
  }

  Widget _pronounCell(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return PronounField(
          targetLanguage: testModel.targetLanguageIso,
          fullPronoun: testModel.pronouns[person],
          expectedIndicative: testModel.expected[person]);
    });
  }

  static Key toTestButtonKey() {
    return const Key('forward_button');
  }
}
