import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_study_form_appbar.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_test_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/theme/custom_theme.dart';
import 'package:cramr_ui_lib/widgets/flash_card_stack.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../widgets/conjugation_meta_data_card.dart';

/// Screen for studying with flash cards.
class VerbIndicativeStudyFlashcardModeRoute extends StatelessWidget {
  static const routeName = '/verbIndicativeFlashcardModeStudyRoute';

  final SettingsController _settingsController;

  const VerbIndicativeStudyFlashcardModeRoute(this._settingsController,
      {Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: _settingsController,
      builder: (BuildContext context, Widget? child) {
        return MaterialApp(
          title: "Verbomatique",
          debugShowCheckedModeBanner: false,
          themeMode: _settingsController.themeMode,
          theme: CustomTheme.lightTheme,
          darkTheme: ThemeData.dark(),
          home: Scaffold(
            key: UniqueKey(),
            appBar: VerbIndicativeStudyFormAppbar().asWidget(context),
            body: Column(
              children: [
                _conjugationMetaDataCard(context),
                const Divider(),
                Expanded(child: _provideFlashCardStack(context)),
              ],
            ),
            floatingActionButton: Visibility(
              key: const Key('flashcard_forward_button_visibility'),
              child: _toTestRouteButton(context),
            ),
          ),
        );
      },
    );
  }

  Widget _provideFlashCardStack(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return FlashCardStack(
          numberOfCards: testModel.items.length,
          expectedIndicative: testModel.expected,
          expectedPronouns: testModel.pronouns);
    });
  }

  Widget _conjugationMetaDataCard(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return ConjugationMetaDataCard(
          infinitive: testModel.infinitive,
          translation: testModel.translation,
          mode: testModel.mode,
          tense: testModel.tense);
    });
  }

  Widget _toTestRouteButton(BuildContext context) {
    return FloatingActionButton(
      key: const Key('flashcard_forward_button'),
      child: const Icon(Icons.arrow_forward),
      onPressed: () {
        final StudyModeState studyModeState =
            Provider.of<StudyModeState>(context, listen: false);
        studyModeState.updateState(StudyMode.flashCardMode);
        Navigator.pushNamed(context, VerbIndicativeTestRoute.routeName);
      },
    );
  }
}
