import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/util/text_size_limiter.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// Stateful widget that allows to show an indicative for a given person, and
/// that allows to cover (default) this indicative.
class CoverableTextField extends StatefulWidget {
  final int person;

  const CoverableTextField({Key? key, required this.person}) : super(key: key);

  @override
  State<CoverableTextField> createState() => CoverableTextFieldState();

  static Key coverKey(String infinitive, int person) {
    return Key('coverkey_${infinitive}_$person');
  }
}

/// State of the field is about whether it is covered or not.
class CoverableTextFieldState extends State<CoverableTextField> {
  bool _coverStatus = true;

  @override
  Widget build(BuildContext context) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return GestureDetector(
        key: Key(
            'cover_gesturedetector_${testModel.infinitive}_${widget.person}'),
        onTap: () {
          setState(() {
            _coverStatus = !_coverStatus;
          });
        },
        child: _coverStatus
            ? Card(
                key: CoverableTextField.coverKey(
                    testModel.infinitive, widget.person),
                child: SizedBox(
                  height: 80,
                  width: 180,
                  child: Container(
                    key: UniqueKey(),
                    color: Colors.green[400],
                  ),
                ),
                elevation: 15.0,
                shadowColor: Colors.green[600],
              )
            : Card(
                key: UniqueKey(),
                child: SizedBox(
                  height: 80,
                  width: 180,
                  child: Container(
                    key: UniqueKey(),
                    alignment: Alignment.center,
                    child: _wrappedConjugation(
                        context, testModel.expected[widget.person]),
                  ),
                ),
                elevation: 10.0,
              ),
      );
    });
  }

  Widget _wrappedConjugation(BuildContext context, String conjugation) {
    List<Text> conjugationParts = [];
    conjugation.split(' ').forEach((element) {
      conjugationParts.add(Text(
        element,
        key: UniqueKey(),
        textDirection: TextDirection.ltr,
        style: const TextStyle(
          color: Colors.green,
          fontSize: 25,
        ),
        overflow: TextOverflow.ellipsis,
        textScaleFactor: TextSizeLimiter.textScaleFactor(context),
      ));
      conjugationParts.add(Text(
        ' ',
        key: UniqueKey(),
        textDirection: TextDirection.ltr,
        style: const TextStyle(
          color: Colors.green,
          fontSize: 25,
        ),
        overflow: TextOverflow.ellipsis,
        textScaleFactor: TextSizeLimiter.textScaleFactor(context),
      ));
    });
    Wrap result = Wrap(
      children: conjugationParts,
    );
    return result;
  }
}
