import 'package:cramr_ui_lib/widgets/pronoun_field.dart';
import 'package:flutter/material.dart';
import '../util/text_size_limiter.dart';

class EvaluationRow extends StatelessWidget {
  const EvaluationRow(
      this.person, this.language, this.fullPronoun, this.actual, this.expected,
      {Key? key})
      : super(key: key);

  final int person;
  final String language;
  final String fullPronoun;
  final String actual;
  final String expected;

  @override
  Widget build(BuildContext context) {
    return Row(children: [
      _pronounCell(context, person, language, fullPronoun, actual),
      Expanded(
        key: UniqueKey(),
        child: Text(
          actual,
          key: UniqueKey(),
          textAlign: TextAlign.right,
          style: TextStyle(
            color: _actualTextColor(actual, expected),
            fontSize: 25,
          ),
          textScaleFactor: TextSizeLimiter.textScaleFactor(context),
        ),
      ),
      Visibility(
        child: SizedBox(
          width: MediaQuery.of(context).size.width * 0.07,
          child: IconButton(
              key: wrongAnswerKey(expected, person),
              onPressed: () {
                _showCorrectAnswerDialog(expected, context);
              },
              icon: const Icon(
                Icons.help_outline_sharp,
              )),
        ),
        visible: (actual != expected),
      )
    ]);
  }

  Color _actualTextColor(String actual, String expected) {
    return (actual == expected) ? Colors.green : Colors.red;
  }

  Widget _pronounCell(BuildContext context, int person, String language,
      String fullPronoun, String expectedIndicative) {
    return PronounField(
        targetLanguage: language,
        fullPronoun: fullPronoun,
        expectedIndicative: expectedIndicative);
  }

  static Key wrongAnswerKey(String expected, int person) {
    return Key('wrong_${expected}_$person');
  }

  Future<void> _showCorrectAnswerDialog(
      String expected, BuildContext context) async {
    return showDialog<void>(
        context: context,
        barrierDismissible: true,
        builder: (BuildContext context) {
          return AlertDialog(
            title: Text(
              "Juiste antwoord",
              key: UniqueKey(),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            content: Text(
              expected,
              key: UniqueKey(),
              style: const TextStyle(
                color: Colors.green,
                fontSize: 25,
              ),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            actions: [
              TextButton(
                key: const Key('got_it_button'),
                onPressed: () {
                  Navigator.of(context).pop();
                },
                child: Text(
                  "Oké. Ik snap het.",
                  key: UniqueKey(),
                  textScaleFactor: TextSizeLimiter.textScaleFactor(context),
                ),
              )
            ],
          );
        });
  }
}
