import 'package:cramr_ui_lib/widgets/text_checker.dart';
import 'package:flutter/material.dart';
import 'expected_row.dart';

/// Stateful widget that allows to browse through an indicative and in the
/// mean time write down the form for practice purposes.
class WrittenIndicativeStudyWidget extends StatefulWidget {
  const WrittenIndicativeStudyWidget(
      {Key? key,
      required this.expectedIndicative,
      required this.fullPronouns,
      required this.targetLanguage,
      required this.numberOfItems})
      : super(key: key);

  final List<String> expectedIndicative;
  final List<String> fullPronouns;
  final String targetLanguage;
  final int numberOfItems;

  @override
  State<WrittenIndicativeStudyWidget> createState() =>
      WrittenIndicativeStudyWidgetState();
}

/// The state to maintain is the person of the indicative that is currently
/// being displayed.
class WrittenIndicativeStudyWidgetState
    extends State<WrittenIndicativeStudyWidget> {
  int _currentPerson = 0;

  @override
  Widget build(BuildContext context) {
    return Card(
      child: Column(
        children: [
          ExpectedRow(
            currentPerson: _currentPerson,
            expectedIndicative: widget.expectedIndicative[_currentPerson],
            fullPronoun: widget.fullPronouns[_currentPerson],
            targetLanguage: widget.targetLanguage,
          ),
          _provideTextChecker(context),
          IconButton(
              key: const Key('nextPersonKey'),
              onPressed: () {
                setState(() {
                  _currentPerson = ++_currentPerson % widget.numberOfItems;
                });
              },
              icon: const Icon(Icons.arrow_forward)),
        ],
      ),
      elevation: 15,
    );
  }

  Widget _provideTextChecker(BuildContext context) {
    return TextChecker(
        currentPerson: _currentPerson,
        key: UniqueKey(),
        language: widget.targetLanguage,
        fullPronoun: widget.fullPronouns[_currentPerson],
        expected: widget.expectedIndicative[_currentPerson]);
  }
}
