import 'package:cramr_ui_lib/model/conjugation.dart';
import 'package:cramr_ui_lib/model/french_verb.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  var _infinitive = 'infinitief';
  var _translation = 'translation';
  var _simplePresentIndicative = ['nul', 'een', 'twee', 'drie', 'vier', 'vijf'];
  var _simplePastIndicative = [
    'oudnul',
    'oudeen',
    'oudtwee',
    'ouddrie',
    'oudvier',
    'oudvijf'
  ];
  var _isoLanguage = 'fr';

  group('FrenchVerb tests simple present indicative', () {
    test('FrenchVerb state tests', () {
      Conjugation _conjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      var model = FrenchVerb(_infinitive, _translation, _conjugation);
      expect(_infinitive, model.infinitive);
      expect(_translation, model.translation);
      expect(_isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in _simplePresentIndicative) {
        expect(element, model.simplePresentIndicative[i++]);
      }
      if (null != model.simplePastIndicative) {
        fail('non null where null expected');
      }
    });

    test('Test for hasSimplePastIndicative', () {
      Conjugation _conjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      var model = FrenchVerb(_infinitive, _translation, _conjugation);
      expect(false, model.hasSimplePastIndicative());
    });
  });

  group('FrenchVerb tests several indicatives', () {
    test('FrenchVerb state tests', () {
      Conjugation _presentConjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      Conjugation _pastConjugation =
          Conjugation('indicatif', 'imparfait', _simplePastIndicative);
      var model = FrenchVerb.fromMoreTensesAndModes(
          _infinitive, _translation, _presentConjugation, _pastConjugation);
      expect(_infinitive, model.infinitive);
      expect(_translation, model.translation);
      expect(_isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in _simplePastIndicative) {
        expect(element, model.simplePastIndicative![i++]);
      }
    });

    test('Test for hasSimplePastIndicative', () {
      Conjugation _presentConjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      Conjugation _pastConjugation =
          Conjugation('indicatif', 'imparfait', _simplePastIndicative);
      var model = FrenchVerb.fromMoreTensesAndModes(
          _infinitive, _translation, _presentConjugation, _pastConjugation);
      expect(true, model.hasSimplePastIndicative());
    });
  });
}
