import 'package:cramr_ui_lib/model/conjugation.dart';
import 'package:cramr_ui_lib/model/verb.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  var _infinitive = 'infinitief';
  var _translation = 'translation';
  var _simplePresentIndicative = ['nul', 'een', 'twee', 'drie', 'vier', 'vijf'];
  var _simplePastIndicative = [
    'oudnul',
    'oudeen',
    'oudtwee',
    'ouddrie',
    'oudvier',
    'oudvijf'
  ];
  var _presentConditional = [
    'condnul',
    'condeen',
    'condtwee',
    'conddrie',
    'condvier',
    'condvijf'
  ];
  var _simpleFutureIndicative = [
    'toekomstnul',
    'toekomsteen',
    'toekomsttwee',
    'toekomstdrie',
    'toekomstvier',
    'toekomstvijf',
  ];
  var _isoLanguage = 'fr';

  group('Verb tests simple present indicative', () {
    test('Verb state tests', () {
      Conjugation _conjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      var model = Verb(_infinitive, _translation, _conjugation, _isoLanguage);
      expect(_infinitive, model.infinitive);
      expect(_translation, model.translation);
      expect(_isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in _simplePresentIndicative) {
        expect(element, model.simplePresentIndicative[i++]);
      }
      if (null != model.simplePastIndicative) {
        fail('non null where null expected');
      }
    });

    test('Test for hasSimplePastIndicative', () {
      Conjugation _conjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      var model = Verb(_infinitive, _translation, _conjugation, _isoLanguage);
      expect(false, model.hasSimplePastIndicative());
    });
  });

  group('Verb tests several indicatives', () {
    test('Verb state tests two tenses', () {
      Conjugation _presentConjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      Conjugation _pastConjugation =
          Conjugation('indicatif', 'imparfait', _simplePastIndicative);
      var model = Verb.fromMoreTensesAndModes(_infinitive, _translation,
          _presentConjugation, _pastConjugation, _isoLanguage);
      expect(_infinitive, model.infinitive);
      expect(_translation, model.translation);
      expect(_isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in _simplePastIndicative) {
        expect(element, model.simplePastIndicative![i++]);
      }
    });

    test('Verb state tests three tenses', () {
      Conjugation _presentConjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      Conjugation _pastConjugation =
          Conjugation('indicatif', 'imparfait', _simplePastIndicative);
      Conjugation _futureConjugation =
          Conjugation('indicatif', 'futur simple', _simpleFutureIndicative);
      var model = Verb.fromAllTensesAndModes(
          _infinitive,
          _translation,
          [_presentConjugation, _pastConjugation, _futureConjugation],
          _isoLanguage);
      expect(_infinitive, model.infinitive);
      expect(_translation, model.translation);
      expect(_isoLanguage, model.isoLanguage);
      int i = 0;
      for (var element in _simplePastIndicative) {
        expect(element, model.simplePastIndicative![i++]);
      }
      i = 0;
      for (var element in _simpleFutureIndicative) {
        expect(element, model.simpleFutureIndicative[i++]);
      }
    });

    test('Test for hasSimplePastIndicative', () {
      Conjugation _presentConjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      Conjugation _pastConjugation =
          Conjugation('indicatif', 'imparfait', _simplePastIndicative);
      var model = Verb.fromMoreTensesAndModes(_infinitive, _translation,
          _presentConjugation, _pastConjugation, _isoLanguage);
      expect(true, model.hasSimplePastIndicative());
    });

    test('Verb tests mixed modes and tenses with duplicates', () {
      Conjugation _presentConjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      Conjugation _pastConjugation =
          Conjugation('conditionnel', 'présent', _presentConditional);
      Conjugation _futureConjugation =
          Conjugation('indicatif', 'futur simple', _simpleFutureIndicative);
      var model = Verb.fromAllTensesAndModes(
          _infinitive,
          _translation,
          [_presentConjugation, _pastConjugation, _futureConjugation],
          _isoLanguage);
      expect(_infinitive, model.infinitive);
      expect(_translation, model.translation);
      expect(_isoLanguage, model.isoLanguage);
      expect(3, model.indicativeConjugations.length);
    });

    test('test the conjugation method', () {
      Conjugation _presentConjugation =
          Conjugation('indicatif', 'présent', _simplePresentIndicative);
      Conjugation _pastConjugation =
          Conjugation('indicatif', 'imparfait', _simplePastIndicative);
      Conjugation _futureConjugation =
          Conjugation('indicatif', 'futur simple', _simpleFutureIndicative);
      var model = Verb.fromAllTensesAndModes(
          _infinitive,
          _translation,
          [_presentConjugation, _pastConjugation, _futureConjugation],
          _isoLanguage);
      expect(_infinitive, model.infinitive);
      expect(_translation, model.translation);
      expect(_isoLanguage, model.isoLanguage);
      expect(model.conjugation('indicatif', 'imparfait')?.tense, 'imparfait');
      expect(model.conjugation('indicatif', 'imparfait')?.mode, 'indicatif');
      expect(model.conjugation('conditionnel', 'imparfait'), isNull);
    });
  });
}
