import 'package:test/test.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';

void main() {
  group('Verb Test Model tests', () {
    var _expectedFull = ["ai", "as", "a", "avons", "avez", "ont"];
    var _expectedSingular = _expectedFull.sublist(0, 2);
    var _expectedPlural = _expectedFull.sublist(3, 5);
    var _pronouns = ["je", "tu", "il", "nous", "vous", "ils"];
    var _pronounsSingular = _pronouns.sublist(0, 2);
    var _pronounsPlural = _pronouns.sublist(3, 5);
    var _targetLanguageIso = "fr";
    var _infinitive = "avoir";
    var _translation = "hebben";
    var _mode = 'indicatif';
    var _tense = 'présent';

    test('Some random behaviour tests for the model', () {
      var model = VerbTestModel();
      model.initialise(_expectedFull, _pronouns, _infinitive,
          _targetLanguageIso, _translation, _mode, _tense);
      model.add("banana", 3);

      expect(model.isComplete(), false);
      expect(model.score, 0);
      expect(model.items.length, 6);
      expect(model.items[0], "");
      expect(model.items[3], "banana");
      expect(model.pronouns.length, _pronouns.length);
      expect(model.pronouns[1], _pronouns[1]);
      expect(model.translation, _translation);
      expect(model.tense, _tense);
      expect(model.mode, _mode);

      model.reset();

      expect(model.items.length, 6);
      expect(model.items[3], "");
      expect(model.translation, _translation);
    });

    test('Test the trimming of the leading spaces', () {
      var model = VerbTestModel();
      model.initialise(_expectedFull, _pronouns, _infinitive,
          _targetLanguageIso, _translation, _mode, _tense);
      model.reset();
      model.add(" leading", 0);
      expect(model.items[0], "leading");
    });

    test('Test the trimming of the trailing spaces', () {
      var model = VerbTestModel();
      model.initialise(_expectedFull, _pronouns, _infinitive,
          _targetLanguageIso, _translation, _mode, _tense);
      model.reset();
      model.add("trailing ", 0);
      expect(model.items[0], "trailing");
    });

    test('test isComplete method, focus on leading and trailing spaces', () {
      var model = VerbTestModel();
      model.initialise(_expectedFull, _pronouns, _infinitive,
          _targetLanguageIso, _translation, _mode, _tense);
      model.reset();
      model.add(" leading", 0);
      expect(model.isComplete(), false);
      model.add("trailing ", 1);
      expect(model.isComplete(), false);
      model.add(" ", 2);
      expect(model.isComplete(), false);
      model.add(" ", 3);
      expect(model.isComplete(), false);
      model.add(" ", 4);
      expect(model.isComplete(), false);
      model.add(" ", 5);
      expect(model.isComplete(), false);

      model.add("something", 3);
      expect(model.isComplete(), false);

      model.add(" haha", 4);
      expect(model.isComplete(), false);

      model.add(" haha", 5);
      expect(model.isComplete(), false);

      model.add("dont forget", 2);
      expect(model.isComplete(), true);
    });

    test('test singular only', () {
      var model = VerbTestModel();
      model.initialise(_expectedSingular, _pronounsSingular, _infinitive,
          _targetLanguageIso, _translation, _mode, _tense);
      expect(model.isComplete(), false);
      expect(model.pronouns.length, _pronounsSingular.length);
      var _pronounCounter = 0;
      for (var element in _pronounsSingular) {
        expect(model.pronouns[_pronounCounter++], element);
      }
      expect(0, model.score);
      var _singularCounter = 0;
      for (var element in _expectedSingular) {
        model.add(element, _singularCounter++);
        expect(model.score, _singularCounter);
      }
      expect(model.isComplete(), true);
    });

    test('test plural only', () {
      var model = VerbTestModel();
      model.initialise(_expectedPlural, _pronounsPlural, _infinitive,
          _targetLanguageIso, _translation, _mode, _tense);
      expect(model.isComplete(), false);
      expect(model.pronouns.length, _pronounsPlural.length);
      var _pronounCounter = 0;
      for (var element in _pronounsPlural) {
        expect(model.pronouns[_pronounCounter++], element);
      }
      expect(0, model.score);
      var _pluralCounter = 0;
      for (var element in _expectedPlural) {
        model.add(element, _pluralCounter++);
        expect(model.score, _pluralCounter);
      }
      expect(model.isComplete(), true);
    });
  });
}
