//TODO: introduce mocking
import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/study_mode.dart';
import 'package:cramr_ui_lib/model/study_mode_state.dart';
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_indicative_test_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_service.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

final List<String> _randomIndicative = [
  'exp1',
  'exp2',
  'exp3',
  'exp4',
  'exp5',
  'exp6',
];

final List<String> _randomPronouns = [
  'pron1',
  'pron2',
  'pron3',
  'pron4',
  'pron5',
  'pron6',
];

const String _randomInfinitive = 'infinitive';
const String _mode = 'mode';
const String _tense = 'tense';

void main() {
  test('Test key generation for textfield', () {
    const Key expectedKey = Key('test_infinitive_1');
    expect(VerbIndicativeTestRoute.testFieldKey(_randomInfinitive, 1),
        expectedKey);
  });

  test('Test key generation for forward button', () {
    const Key expectedKey = Key('evaluate_infinitive');
    expect(VerbIndicativeTestRoute.toEvaluationButtonKey(_randomInfinitive),
        expectedKey);
  });

  testWidgets('Testing for presence of all fields. Some logic too.',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(_randomIndicative, _randomPronouns, 'infinitive',
                'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.coverMode);
          })
        ],
        child: MaterialApp(
          home: VerbIndicativeTestRoute(
            SettingsController(SettingsService(prefs), ThemeMode.dark, prefs),
            key: UniqueKey(),
          ),
        )));

    Finder titleFinderOne = find.text('Test je kennis');
    expect(titleFinderOne, findsOneWidget);
    Finder forwardButtonFinder =
        find.byKey(const Key('evaluate_$_randomInfinitive'));
    expect(forwardButtonFinder, findsOneWidget);
    await tester.tap(forwardButtonFinder);
    await tester.pumpAndSettle();
    expect(titleFinderOne, findsOneWidget);
    int i = 0;
    for (var element in _randomIndicative) {
      final Finder textFieldFinder = find
          .byKey(VerbIndicativeTestRoute.testFieldKey(_randomInfinitive, i));
      expect(textFieldFinder, findsOneWidget);
      await tester.enterText(textFieldFinder, element);
      await tester.pumpAndSettle();
      i++;
    }
  });

  testWidgets('Testing for presence of singular fields. Some logic too.',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(true, false);
    var filteredPronounList =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    var filteredIndicativeList =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(filteredIndicativeList, filteredPronounList,
                'infinitive', 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.coverMode);
          })
        ],
        child: MaterialApp(
          home: VerbIndicativeTestRoute(
            SettingsController(SettingsService(prefs), ThemeMode.dark, prefs),
            key: UniqueKey(),
          ),
        )));

    Finder titleFinderOne = find.text('Test je kennis');
    expect(titleFinderOne, findsOneWidget);
    Finder forwardButtonFinder = find.byKey(
        VerbIndicativeTestRoute.toEvaluationButtonKey(_randomInfinitive));
    expect(forwardButtonFinder, findsOneWidget);
    await tester.tap(forwardButtonFinder);
    await tester.pumpAndSettle();
    expect(titleFinderOne, findsOneWidget);
    int i = 0;
    for (var element in filteredIndicativeList) {
      final Finder textFieldFinder = find
          .byKey(VerbIndicativeTestRoute.testFieldKey(_randomInfinitive, i));
      expect(textFieldFinder, findsOneWidget);
      await tester.enterText(textFieldFinder, element);
      await tester.pumpAndSettle();
      i++;
    }
  });

  testWidgets('Testing for presence of plural fields. Some logic too.',
      (WidgetTester tester) async {
    SharedPreferences.setMockInitialValues(<String, Object>{'counter': 1});
    SharedPreferences prefs = await SharedPreferences.getInstance();
    var selection = CardinalitySelection(false, true);
    var filteredPronounList =
        CardinalityFilter.filteredList(selection, _randomPronouns);
    var filteredIndicativeList =
        CardinalityFilter.filteredList(selection, _randomIndicative);
    await tester.pumpWidget(MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (BuildContext context) {
            var model = VerbTestModel();
            model.initialise(filteredIndicativeList, filteredPronounList,
                'infinitive', 'fr', 'translation', _mode, _tense);
            return model;
          }),
          ChangeNotifierProvider(create: (BuildContext context) {
            return StudyModeState(StudyMode.coverMode);
          })
        ],
        child: MaterialApp(
          home: VerbIndicativeTestRoute(
            SettingsController(SettingsService(prefs), ThemeMode.dark, prefs),
            key: UniqueKey(),
          ),
        )));

    Finder titleFinderOne = find.text('Test je kennis');
    expect(titleFinderOne, findsOneWidget);
    Finder forwardButtonFinder = find.byKey(
        VerbIndicativeTestRoute.toEvaluationButtonKey(_randomInfinitive));
    expect(forwardButtonFinder, findsOneWidget);
    await tester.tap(forwardButtonFinder);
    await tester.pumpAndSettle();
    expect(titleFinderOne, findsOneWidget);
    int i = 0;
    for (var element in filteredIndicativeList) {
      final Finder textFieldFinder = find
          .byKey(VerbIndicativeTestRoute.testFieldKey(_randomInfinitive, i));
      expect(textFieldFinder, findsOneWidget);
      await tester.enterText(textFieldFinder, element);
      await tester.pumpAndSettle();
      i++;
    }
  });
}
