import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/util/cardinality_filter.dart';
import 'package:flutter_test/flutter_test.dart';

main() {
  group('All cardinality tests for the filter utility', () {
    var _pronouns = ['nul', 'een', 'twee', 'drie', 'vier', 'vijf'];
    var _indicative = ['nul', 'een', 'twee', 'drie', 'vier', 'vijf'];

    test('Test singular only', () {
      CardinalitySelection selection = CardinalitySelection(true, false);
      var adaptedIndicativeList =
          CardinalityFilter.filteredList(selection, _indicative);
      var adaptedPronounList =
          CardinalityFilter.filteredList(selection, _pronouns);
      expect(adaptedIndicativeList.length, 3);
      expect(adaptedPronounList.length, 3);
      int i = 0;
      for (var element in adaptedIndicativeList) {
        expect(element, _indicative[i++]);
      }
      expect(i, 3);
      i = 0;
      for (var element in adaptedPronounList) {
        expect(element, _pronouns[i++]);
      }
      expect(i, 3);
    });
    test('Test plural only', () {
      CardinalitySelection selection = CardinalitySelection(false, true);
      var adaptedIndicativeList =
          CardinalityFilter.filteredList(selection, _indicative);
      var adaptedPronounList =
          CardinalityFilter.filteredList(selection, _pronouns);
      expect(adaptedIndicativeList.length, 3);
      expect(adaptedPronounList.length, 3);
      int i = 3;
      for (var element in adaptedIndicativeList) {
        expect(element, _indicative[i++]);
      }
      expect(i, 6);
      i = 3;
      for (var element in adaptedPronounList) {
        expect(element, _pronouns[i++]);
      }
      expect(i, 6);
    });
    test('Test singular and plural', () {
      CardinalitySelection selection = CardinalitySelection(true, true);
      var adaptedIndicativeList =
          CardinalityFilter.filteredList(selection, _indicative);
      var adaptedPronounList =
          CardinalityFilter.filteredList(selection, _pronouns);
      expect(adaptedIndicativeList.length, 6);
      expect(adaptedPronounList.length, 6);
      int i = 0;
      for (var element in adaptedIndicativeList) {
        expect(element, _indicative[i++]);
      }
      expect(i, 6);
      i = 0;
      for (var element in adaptedPronounList) {
        expect(element, _pronouns[i++]);
      }
      expect(i, 6);
    });
  });
}
